#!/usr/bin/env python3
import rospy
import numpy as np  # you probably gonna need this
import sys

from geometry_msgs.msg import Twist, Vector3
from std_srvs.srv import Empty, EmptyResponse
from gazebo_msgs.srv import DeleteLight
import termios
import tty
from select import select


class Robocop:
    def __init__(self):
        rospy.init_node("robocop_keyboard_control")
        self.key_time_wait = 0.1
        self.speed_linear = 0
        self.speed_angular = 0
        # TODO add your own parameters such as speed limits nad how much the speeds should change when key pressed

        self.terminal_settings = termios.tcgetattr(
            sys.stdin
        )  # save terminal setting for later use with key press listening

        # TODO create pub_cmd publisher to "/cmd_vel" topic, find out the message type e.g. by using rostopic info when simulation is running

        # service to stop the robot
        self.stop_service = rospy.Service("stop", Empty, self.stop_service_callback)

        rospy.on_shutdown(self.full_stop)  # function to run when node is killed
        self.key_press_callback()  # blocking function to run the keyboard control

    def key_press_callback(self):
        """method that continuously listen for the key pressed"""

        while not rospy.is_shutdown():  # loop until node is killed

            # loading the pressed keys with some terminal magic
            tty.setraw(sys.stdin.fileno())
            rlist, _, _ = select([sys.stdin], [], [], self.key_time_wait)
            last_key = ""
            if rlist:
                last_key = sys.stdin.read(1)
            termios.tcsetattr(sys.stdin, termios.TCSADRAIN, self.terminal_settings)

            # TODO fill the individual reactions to the pressed keys
            if last_key == "w":
                # TODO react to w key pressed
                rospy.loginfo("w pressed")
            elif last_key == "x":
                # TODO react to x key pressed
                rospy.loginfo("x pressed")
            elif last_key == "a":
                # TODO react to a key pressed
                rospy.loginfo("a pressed")
            elif last_key == "d":
                # TODO react to d key pressed
                rospy.loginfo("d pressed")
            elif last_key == "s":
                self.speed_linear = 0
                self.speed_angular = 0
                rospy.loginfo("s pressed")

            # TODO publish the speeds using the pub_cmd publisher created before

            if last_key == "\x03":
                rospy.signal_shutdown("killed by user")
                break

    def stop_service_callback(self, req):
        """callback to stop service"""
        rospy.loginfo("stopping service received")
        self.full_stop()
        return EmptyResponse()

    def full_stop(self):
        """method to stop the movement"""
        self.speed_linear = 0
        self.speed_angular = 0
        self.pub_cmd.publish(
            Twist(
                linear=Vector3(x=self.speed_linear),
                angular=Vector3(z=self.speed_angular),
            )
        )


if __name__ == "__main__":
    rc = Robocop()
    rospy.spin()
